import datetime

from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.template import loader, Context
from django.views.generic.base import TemplateView, RedirectView
from django.views.generic.list import ListView
from django.contrib import messages

from emissions.models import *
from emissions.utils import period_program

import emissions.views
import newsletter.views

class Home(TemplateView):
    template_name = 'home.html'

    def get_context_data(self, **kwargs):
        context = super(Home, self).get_context_data(**kwargs)
        context['emissions'] = self.request.user.emissions.all().order_by('title')
        context['news_categories'] = self.request.user.news_categories.all().order_by('title')
        if self.request.user.has_perm('emissions.add_focus'):
            context['focused_items'] = Focus.objects.select_related().filter(current=True)
            context['recent_episodes'] = Episode.objects.select_related(
                    ).order_by('-last_update_timestamp')
            context['recent_newsitems'] = NewsItem.objects.select_related(
                    ).order_by('-last_update_timestamp')
            context['recent_soundfiles'] = SoundFile.objects.select_related(
                    ).order_by('-last_update_timestamp')

        return context

home = login_required(Home.as_view())


class EmissionListView(emissions.views.EmissionListView):
    pass

emissions_list = login_required(EmissionListView.as_view())



class NewsListView(ListView):
    model = NewsCategory

    def get_queryset(self):
        return NewsCategory.objects.order_by('title')

    def get_context_data(self, **kwargs):
        context = super(NewsListView, self).get_context_data(**kwargs)
        context['unsorted_newsitems'] = NewsItem.objects.filter(category__isnull=True).order_by('title')
        return context

news = login_required(NewsListView.as_view())


class NewsletterCreateView(newsletter.views.NewsletterCreateView):
    def get_initial(self):
        initial = super(NewsletterCreateView, self).get_initial()

        newsletter_body = loader.get_template('newsletter/body.html')
        newsletter_subject = loader.get_template('newsletter/subject.txt')
        context = Context()

        date = datetime.datetime.today()
        if date.weekday() > 0:
            date = date + datetime.timedelta(days=7-date.weekday())

        date = datetime.datetime(*date.timetuple()[:3])
        date_end = date + datetime.timedelta(days=7)

        context['date'] = date
        context['date_end'] = date_end

        context['soundfiles'] = SoundFile.objects.filter(
                podcastable=True, fragment=False).select_related().filter(
                        creation_timestamp__gte=date-datetime.timedelta(days=7))

        context['diffusions'] = list(Diffusion.objects.select_related().filter(
                datetime__gte=date,
                datetime__lte=date+datetime.timedelta(days=7)).order_by('datetime'))

        # remove rebroadcasts made during the same week
        seen_episodes = {}
        for diff in context['diffusions'][:]:
            if diff.episode.id in seen_episodes:
                context['diffusions'].remove(diff)
            seen_episodes[diff.episode.id] = True

        initial['subject'] = newsletter_subject.render(context)
        initial['text'] = newsletter_body.render(context)
        return initial

newsletter_create = NewsletterCreateView.as_view()


class FocusSetView(RedirectView):
    permanent = False

    def get_redirect_url(self, object_type, object_id):
        if not self.request.user.has_perm('emissions.add_focus'):
            raise PermissionDenied()
        messages.success(self.request, emissions.views.SUCCESS_MESSAGE)
        if object_type == 'emission':
            emission = Emission.objects.get(id=object_id)
            try:
                f = Focus.objects.get(emission=emission)
            except Focus.DoesNotExist:
                f = Focus()
            f.current = True
            f.emission = emission
            f.save()
            return reverse('emission-view', kwargs={'slug': emission.slug})
        if object_type == 'episode':
            episode = Episode.objects.get(id=object_id)
            try:
                f = Focus.objects.get(episode=episode)
            except Focus.DoesNotExist:
                f = Focus()
            f.current = True
            f.episode = episode
            f.save()
            return reverse('episode-view', kwargs={
                'emission_slug': episode.emission.slug, 'slug': episode.slug})
        if object_type == 'newsitem':
            newsitem = NewsItem.objects.get(id=object_id)
            try:
                f = Focus.objects.get(newsitem=newsitem)
            except Focus.DoesNotExist:
                f = Focus()
            f.current = True
            f.newsitem = newsitem
            f.save()
            return reverse('newsitem-view', kwargs={'slug': newsitem.slug})
        if object_type == 'soundfile':
            soundfile = SoundFile.objects.get(id=object_id)
            try:
                f = Focus.objects.get(soundfile=soundfile)
            except Focus.DoesNotExist:
                f = Focus()
            f.current = True
            f.soundfile = soundfile
            f.save()
            return reverse('episode-view', kwargs={
                'slug': soundfile.episode.slug,
                'emission_slug': soundfile.episode.emission.slug,
                })

focus_set = FocusSetView.as_view()


class FocusUnsetView(RedirectView):
    permanent = False

    def get_redirect_url(self, object_type, object_id):
        if not self.request.user.has_perm('emissions.delete_focus'):
            raise PermissionDenied()
        messages.success(self.request, emissions.views.SUCCESS_MESSAGE)
        if object_type == 'emission':
            emission = Emission.objects.get(id=object_id)
            focus = Focus.objects.get(emission=emission)
            focus.current = False
            focus.save()
            return reverse('emission-view', kwargs={'slug': emission.slug})
        if object_type == 'episode':
            episode = Episode.objects.get(id=object_id)
            focus = Focus.objects.get(episode=episode)
            focus.current = False
            focus.save()
            return reverse('episode-view', kwargs={
                'emission_slug': episode.emission.slug, 'slug': episode.slug})
        if object_type == 'newsitem':
            newsitem = NewsItem.objects.get(id=object_id)
            focus = Focus.objects.get(newsitem=newsitem)
            focus.current = False
            focus.save()
            return reverse('newsitem-view', kwargs={'slug': newsitem.slug})
        if object_type == 'soundfile':
            soundfile = SoundFile.objects.get(id=object_id)
            focus = Focus.objects.get(soundfile=soundfile)
            focus.current = False
            focus.save()
            return reverse('episode-view', kwargs={
                'slug': soundfile.episode.slug,
                'emission_slug': soundfile.episode.emission.slug,
                })

focus_unset = FocusUnsetView.as_view()
